"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsDocRuleDefaultOptions = void 0;
const typescript_1 = require("typescript");
const checkSymbolmportability_1 = require("../core/checkSymbolmportability");
const jsdocRule = {
    meta: {
        type: "problem",
        docs: {
            description: "Prohibit importing private exports.",
            url: "TODO",
        },
        messages: {
            "no-program": "Cannot retrieve TypeScript program for this file.",
            package: "Cannot import a package-private export '{{ identifier }}'",
            "package:reexport": "Cannot re-export a package-private export '{{ identifier }}'",
            private: "Cannot import a private export '{{ identifier }}'",
            "private:reexport": "Cannot re-export a private export '{{ identifier }}'",
        },
        schema: [
            {
                type: "object",
                properties: {
                    indexLoophole: {
                        type: "boolean",
                    },
                    filenameLoophole: {
                        type: "boolean",
                    },
                    defaultImportability: {
                        type: "string",
                        enum: ["public", "package", "private"],
                    },
                    treatSelfReferenceAs: {
                        type: "string",
                        enum: ["external", "internal"],
                    }
                },
                additionalProperties: false,
            },
        ],
    },
    defaultOptions: [
        {
            indexLoophole: true,
            filenameLoophole: false,
            defaultImportability: "public",
            treatSelfReferenceAs: "external",
        },
    ],
    create(context) {
        const { parserServices, options } = context;
        if (!parserServices) {
            return {};
        }
        const { indexLoophole, filenameLoophole, defaultImportability, treatSelfReferenceAs, } = jsDocRuleDefaultOptions(options[0]);
        const packageOptions = {
            indexLoophole,
            filenameLoophole,
            defaultImportability,
            treatSelfReferenceAs,
        };
        return {
            ImportSpecifier(node) {
                const sourceFilename = context.getFilename();
                if (!sourceFilename) {
                    return;
                }
                if (parserServices.program === null) {
                    context.report({
                        node,
                        messageId: "no-program",
                    });
                    return;
                }
                const checker = parserServices.program.getTypeChecker();
                // const shouldSkip = shouldSkipSymbolCheck(node, sourceFilename);
                // if (shouldSkip) {
                //   return;
                // }
                parserServices;
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                const symbol = checker.getSymbolAtLocation(tsNode.name);
                if (symbol) {
                    const moduleSpecifier = tsNode.parent.parent.parent.moduleSpecifier;
                    if (!(0, typescript_1.isStringLiteral)(moduleSpecifier)) {
                        // Should not happen (as of TS 5.1)
                        return;
                    }
                    checkSymbol(context, packageOptions, parserServices.program, node, tsNode, moduleSpecifier.text, symbol);
                }
            },
            ImportDefaultSpecifier(node) {
                const sourceFilename = context.getFilename();
                if (!sourceFilename) {
                    return;
                }
                if (parserServices.program === null) {
                    context.report({
                        node,
                        messageId: "no-program",
                    });
                    return;
                }
                const checker = parserServices.program.getTypeChecker();
                // const shouldSkip = shouldSkipSymbolCheck(node, sourceFilename);
                // if (shouldSkip) {
                //   return;
                // }
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                if (!tsNode.name) {
                    return;
                }
                const symbol = checker.getSymbolAtLocation(tsNode.name);
                if (symbol) {
                    const moduleSpecifier = tsNode.parent.moduleSpecifier;
                    if (!(0, typescript_1.isStringLiteral)(moduleSpecifier)) {
                        // Should not happen (as of TS 5.1)
                        return;
                    }
                    checkSymbol(context, packageOptions, parserServices.program, node, tsNode, moduleSpecifier.text, symbol);
                }
            },
            ExportSpecifier(node) {
                const sourceFilename = context.getFilename();
                if (!sourceFilename) {
                    return;
                }
                if (parserServices.program === null) {
                    context.report({
                        node,
                        messageId: "no-program",
                    });
                    return;
                }
                const checker = parserServices.program.getTypeChecker();
                // const shouldSkip = shouldSkipSymbolCheck(node, sourceFilename);
                // if (shouldSkip) {
                //   return;
                // }
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                const symbol = checker.getSymbolAtLocation(tsNode.name);
                if (symbol) {
                    const moduleSpecifier = tsNode.parent.parent.moduleSpecifier;
                    if (!moduleSpecifier || !(0, typescript_1.isStringLiteral)(moduleSpecifier)) {
                        return;
                    }
                    checkSymbol(context, packageOptions, parserServices.program, node, tsNode, moduleSpecifier.text, symbol, true);
                }
            },
        };
    },
};
exports.default = jsdocRule;
function jsDocRuleDefaultOptions(options) {
    const { indexLoophole = true, filenameLoophole = false, defaultImportability = "public", treatSelfReferenceAs = "external", } = options || {};
    return {
        indexLoophole,
        filenameLoophole,
        defaultImportability,
        treatSelfReferenceAs,
    };
}
exports.jsDocRuleDefaultOptions = jsDocRuleDefaultOptions;
function checkSymbol(context, packageOptions, program, originalNode, tsNode, moduleSpecifier, symbol, reexport = false) {
    const checker = program.getTypeChecker();
    const exsy = checker.getImmediateAliasedSymbol(symbol);
    if (!exsy) {
        return;
    }
    const checkResult = (0, checkSymbolmportability_1.checkSymbolImportability)(packageOptions, program, tsNode.getSourceFile().fileName, moduleSpecifier, exsy);
    switch (checkResult) {
        case "package": {
            context.report({
                node: originalNode,
                messageId: reexport ? "package:reexport" : "package",
                data: {
                    identifier: exsy.name,
                },
            });
            break;
        }
        case "private": {
            context.report({
                node: originalNode,
                messageId: reexport ? "private:reexport" : "private",
                data: {
                    identifier: exsy.name,
                },
            });
            break;
        }
    }
}
